#ifndef YANOS_H                                  
#define YANOS_H                                  
/*---------------------------------------------------------------------------*/
/*  OS Definitions                                                           */
/*-------------------------------------------------------------------------->>>
                                                              end of comment */
#define OS_SIGNAL_ACTIVATE_GUI            0x0001
#define OS_SIGNAL_ACTIVATE_INTERPRETER    0x0002
#define OS_SIGNAL_STORE_NV_RAM            0x0004
#define OS_SIGNAL_SPARE4                  0x0008
#define OS_SIGNAL_SPARE5                  0x0010
#define OS_SIGNAL_PING                    0x0020
#define OS_SIGNAL_PONG                    0x0040
#define OS_SIGNAL_SPARE8                  0x0080
#define OS_SIGNAL_AUDIO_OUT_FINISHED      0x0100
#define OS_SIGNAL_BUTTON_PRESSED          0x0200
#define OS_SIGNAL_SPARE11                 0x0400
#define OS_SIGNAL_SPARE12                 0x0800

#define OS_SEMA_AUDIO                     0x0001
#define OS_SEMA_IIC                       0x0002
#define OS_SEMA_RTC                       0x0002
#define OS_SEMA_FRAM                      0x0002
#define OS_SEMA_BLUETOOTH                 0x0004
#define OS_SEMA_RS232                     0x0008
#define OS_SEMA_SDIO                      0x0010
#define OS_SEMA_BEEPER                    0x0020
#define OS_SEMA_SPARE2                    0x0040
#define OS_SEMA_SPARE3                    0x0080
#define OS_SEMA_SPARE4                    0x0100
#define OS_SEMA_SPARE5                    0x0200

#define OS_SEMA_WAS_FREE 0
#define OS_SEMA_IS_LOCKED 1
#define OS_SIGNAL_FAILED 0
#define OS_SIGNAL_GET_DIRECT 1
#define OS_SIGNAL_GET_IN_TIME 2

/*---------------------------------------------------------------------------*/
/*  OS Task Definitions                                                      */
/*-------------------------------------------------------------------------->>>
TASK_NUM must set to the highest task id plus one. Id must start with 1 (idle
is 0) and incremented by 1 (no gap allowed).
Take care of the stacks. Calculate 20KB minus STACKSIZE minus ISTACKSIZE minus
extern variables minus yanos variables minus the fast near variables used by
interptreter and gui -> this size is left for OS stacks.
If the RAM goes very scarce consider also that each task stack will incremented
by 0x20 at create time to have space for the task context.
                                                              end of comment */
// create tasks and stacks                                                              
#define OS_CREATE_ALL_TASKS \
  OS_CREATE_TASK(os_tsk_gui,0x1000);\
  OS_CREATE_TASK(os_tsk_interpreter,0x80);\
  OS_CREATE_TASK(os_tsk_ping,0x400);\
  OS_CREATE_TASK(os_tsk_pong,0x400);\
  OS_CREATE_TASK(os_tsk_rs232,0x40);\
  OS_CREATE_TASK(os_tsk_event,0x40);\
  OS_CREATE_TASK(os_tsk_friendly,0x60);

// initialize stack pointer and task entry, set task id
#define OS_INIT_ALL_TASKS \
  OS_INIT_TASK(1,os_tsk_gui);\
  OS_INIT_TASK(2,os_tsk_interpreter);\
  OS_INIT_TASK(3,os_tsk_ping);\
  OS_INIT_TASK(4,os_tsk_pong);\
  OS_INIT_TASK(5,os_tsk_rs232);\
  OS_INIT_TASK(6,os_tsk_event);\
  OS_INIT_TASK(7,os_tsk_friendly);

// TBD define an automation for this
#define TASK_NUM 8

/*---------------------------------------------------------------------------*/
/*  OS Interface - Privat                                                    */
/*-------------------------------------------------------------------------->>>
Do not use in appliction
                                                              end of comment */
#define OS_StartScheduler() tb2s=1
#define OS_StopScheduler() tb2s=0                              
void OS_Init(void);
void OS_TimerDec(void);

/*---------------------------------------------------------------------------*/
/*  OS Interface - void OS_TaskDelay(U16)                                    */
/*-------------------------------------------------------------------------->>>
Description: Delay (stop) the task for the given number of timer ticks
             eg: there are 2 not cooparative tasks, one task delays itself
             for 100 ms. For this time the other task get the rest of the
             processor time.
             Pay attention that three realy realtime tasks run in the backround.
             So this task will get only appr 98% instead of 100%
             (maybe less if there is CAN traffic or someone penetrates the touch)
Application: OS_TaskDelay(500); // delay for half a second
                                                              end of comment */
void OS_TaskDelay(U16);
/*---------------------------------------------------------------------------*/
/*  OS Interface - void OS_TaskSetFriendlyLevel(U16)                         */
/*-------------------------------------------------------------------------->>>
Description: A task can decide by itself to be friendly to the other tasks.
             The FriendlyLevel is the quantity of own timeslots which the task 
             voluntary leave for the others.
             eg1: there are 2 not cooparative tasks - running roughly 50-50
             eg2: there are 3 not cooparative tasks - running roughly 33-33-33
             eg3: one nocoop task, one friendly(1)  - running roughly 66-33
             eg4: one nocoop task, one friendly(5)  - running roughly 80-20
             eg5: one friendly(1), one friendly(1)  - roughly 50-50 but makes no sense
Application: OS_TaskSetFriendlyLevel(0); // take all I can
                                                              end of comment */
void OS_TaskSetFriendlyLevel(U16);
/*---------------------------------------------------------------------------*/
/*  OS Interface - void OS_SetSignal(U16)                                    */
/*-------------------------------------------------------------------------->>>
Description: Produce a signal
Application: OS_SetSignal(OS_SIGNAL_SET_RTC); // request to set the clock
                                                              end of comment */
void OS_SetSignal(U16);
/*---------------------------------------------------------------------------*/
/*  OS Interface - void OS_WaitSignal(U16)                                   */
/*-------------------------------------------------------------------------->>>
Description: Wait for a signal
             eg1: one nocoop task, one waiting - running roughly 100-0
             eg2: one friendly(5), one nocoop, one waiting - roughly 20-80-0
Application: OS_WaitSignal(OS_SIGNAL_ALARM_CLOCK); // something happened
                                                              end of comment */
void OS_WaitSignal(U16);
/*---------------------------------------------------------------------------*/
/*  OS Interface - U16 OS_WaitSignalWithTimeout(U16 sig, S16 tout)           */
/*-------------------------------------------------------------------------->>>
TBD das ist noch nicht getestet
Description: Do not wait for eternity
Application: result=OS_WaitSignal(OS_SIGNAL_ALARM_CLOCK,5000);
             returns OS_SIGNAL_GET_DIRECT if signal is already produced
             returns OS_SIGNAL_GET_IN_TIME if signal received in time
             returns OS_SIGNAL_FAILED if no signal received in timeout time
                                                              end of comment */
U16 OS_WaitSignalWithTimeout(U16 sig, S16 tout);
/*---------------------------------------------------------------------------*/
/*  OS Interface - void OS_GetSema(U16)                                      */
/*-------------------------------------------------------------------------->>>
Description: If sema is locked wait until sema becomes available
             If sema is free then lock it and return
Application: OS_GetSema(OS_SEMA_MySema);
                                                              end of comment */
void OS_GetSema(U16);
/*---------------------------------------------------------------------------*/
/*  OS Interface - void OS_ReleaseSema(U16)                                  */
/*-------------------------------------------------------------------------->>>
Description: Release the sema (unlock it). If there is another task in the
             round waiting for this sema make (only) the next one runable.
             GetSema and ReleaseSema must be used in pairs.
             Exeption: We do not realy know how long the output of a wavefile
             take. Busy waiting to release the sema afterwards makes no sense.
             In this case the sema will released in the interrupt routine
             when the DMA transfer is finished.
Application: OS_ReleaseSema(OS_SEMA_MySema);
                                                              end of comment */
void OS_ReleaseSema(U16);
/*---------------------------------------------------------------------------*/
/*  OS Interface - U16 OS_SemaInquiry(U16)                                   */
/*-------------------------------------------------------------------------->>>
Description: Check if a sema is available
             returns OS_SEMA_WAS_FREE but now it is locked by us
             returns OS_SEMA_IS_LOCKED already by another
Application: if (OS_SemaInquiry(OS_SEMA_MySema) == OS_SEMA_IS_LOCKED) {
               bad_luck();
             } else {
               use_MySema();
               OS_ReleaseSema(OS_SEMA_MySema);
             }
                                                              end of comment */
U16 OS_SemaInquiry(U16);


#else
 #pragma WARN Header was already included
#endif


